<?php

namespace Bixo\Developers\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Bixo\Developers\Http\Requests\DevelopersRequest;
use Bixo\Developers\Interfaces\DevelopersRepositoryInterface;
use Bixo\Developers\Models\Developers;

/**
 * Resource controller class for developers.
 */
class DevelopersResourceController extends BaseController
{

    /**
     * Initialize developers resource controller.
     *
     * @param type DevelopersRepositoryInterface $developers
     *
     * @return null
     */
    public function __construct(DevelopersRepositoryInterface $developers)
    {
        parent::__construct();
        $this->repository = $developers;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Developers\Repositories\Criteria\DevelopersResourceCriteria::class);
    }

    /**
     * Display a list of developers.
     *
     * @return Response
     */
    public function index(DevelopersRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Bixo\Developers\Repositories\Presenter\DevelopersPresenter::class)
                ->$function();
        }

        $developers = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('developers::developers.names'))
            ->view('developers::developers.index', true)
            ->data(compact('developers', 'view'))
            ->output();
    }

    /**
     * Display developers.
     *
     * @param Request $request
     * @param Model   $developers
     *
     * @return Response
     */
    public function show(DevelopersRequest $request, Developers $developers)
    {

        if ($developers->exists) {
            $view = 'developers::developers.show';
        } else {
            $view = 'developers::developers.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('developers::developers.name'))
            ->data(compact('developers'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new developers.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(DevelopersRequest $request)
    {

        $developers = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('developers::developers.name')) 
            ->view('developers::developers.create', true) 
            ->data(compact('developers'))
            ->output();
    }

    /**
     * Create new developers.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(DevelopersRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $developers                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('developers::developers.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('developers/developers/' . $developers->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/developers/developers'))
                ->redirect();
        }

    }

    /**
     * Show developers for editing.
     *
     * @param Request $request
     * @param Model   $developers
     *
     * @return Response
     */
    public function edit(DevelopersRequest $request, Developers $developers)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('developers::developers.name'))
            ->view('developers::developers.edit', true)
            ->data(compact('developers'))
            ->output();
    }

    /**
     * Update the developers.
     *
     * @param Request $request
     * @param Model   $developers
     *
     * @return Response
     */
    public function update(DevelopersRequest $request, Developers $developers)
    {
        try {
            $attributes = $request->all();

            $developers->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('developers::developers.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('developers/developers/' . $developers->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('developers/developers/' . $developers->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the developers.
     *
     * @param Model   $developers
     *
     * @return Response
     */
    public function destroy(DevelopersRequest $request, Developers $developers)
    {
        try {

            $developers->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('developers::developers.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('developers/developers/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('developers/developers/' . $developers->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple developers.
     *
     * @param Model   $developers
     *
     * @return Response
     */
    public function delete(DevelopersRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('developers::developers.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('developers/developers'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/developers/developers'))
                ->redirect();
        }

    }

    /**
     * Restore deleted developers.
     *
     * @param Model   $developers
     *
     * @return Response
     */
    public function restore(DevelopersRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('developers::developers.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/developers/developers'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/developers/developers/'))
                ->redirect();
        }

    }

}
